(function (global, factory) {
    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('rxjs/operators'), require('@angular/core'), require('@ng-web-apis/common'), require('rxjs')) :
    typeof define === 'function' && define.amd ? define('@ng-web-apis/midi', ['exports', 'rxjs/operators', '@angular/core', '@ng-web-apis/common', 'rxjs'], factory) :
    (global = typeof globalThis !== 'undefined' ? globalThis : global || self, factory((global["ng-web-apis"] = global["ng-web-apis"] || {}, global["ng-web-apis"].midi = {}), global.rxjs.operators, global.ng.core, global.common, global.rxjs));
})(this, (function (exports, operators, i0, common, rxjs) { 'use strict';

    function _interopNamespace(e) {
        if (e && e.__esModule) return e;
        var n = Object.create(null);
        if (e) {
            Object.keys(e).forEach(function (k) {
                if (k !== 'default') {
                    var d = Object.getOwnPropertyDescriptor(e, k);
                    Object.defineProperty(n, k, d.get ? d : {
                        enumerable: true,
                        get: function () { return e[k]; }
                    });
                }
            });
        }
        n["default"] = e;
        return Object.freeze(n);
    }

    var i0__namespace = /*#__PURE__*/_interopNamespace(i0);

    function between(value, min, max) {
        return value >= min && value <= max;
    }

    /**
     * Filter MIDI messages to aftertouch changes only
     */
    function aftertouch() {
        return function (source) { return source.pipe(operators.filter(function (_a) {
            var data = _a.data;
            return between(data[0], 208, 223);
        })); };
    }

    /**
     * Filter MIDI messages by channel
     *
     * @param channel number from 0 to 15
     */
    function filterByChannel(channel) {
        return function (source) { return source.pipe(operators.filter(function (_a) {
            var data = _a.data;
            return data[0] % 16 === channel;
        })); };
    }

    /**
     * Filter MIDI messages by MIDIInput id
     *
     * @param id
     */
    function filterById(id) {
        return function (source) { return source.pipe(operators.filter(function (_a) {
            var target = _a.target;
            return target.id === id;
        })); };
    }

    /**
     * Filter MIDI messages by MIDIInput name
     *
     * @param name
     */
    function filterByName(name) {
        return function (source) { return source.pipe(operators.filter(function (_a) {
            var target = _a.target;
            return target.name === name;
        })); };
    }

    /**
     * Filter MIDI messages to main volume changes only
     */
    function mainVolume() {
        return function (source) { return source.pipe(operators.filter(function (_a) {
            var data = _a.data;
            return between(data[0], 176, 191) && data[1] === 7;
        })); };
    }

    /**
     * Filter MIDI messages to modulation wheel changes only
     */
    function modulationWheel() {
        return function (source) { return source.pipe(operators.filter(function (_a) {
            var data = _a.data;
            return between(data[0], 176, 191) && data[1] === 1;
        })); };
    }

    /**
     * Filter MIDI messages to notes only
     *
     * IMPORTANT: It normalizes noteOff events to noteOn with 0 velocity
     */
    function notes() {
        return function (source) { return source.pipe(operators.filter(function (_a) {
            var data = _a.data;
            return between(data[0], 128, 159);
        }), operators.map(function (event) {
            if (between(event.data[0], 128, 143)) {
                event.data[0] += 16;
                event.data[2] = 0;
            }
            return event;
        })); };
    }

    /**
     * Filter MIDI messages to pan changes only
     */
    function pan() {
        return function (source) { return source.pipe(operators.filter(function (_a) {
            var data = _a.data;
            return between(data[0], 176, 191) && data[1] === 10;
        })); };
    }

    /**
     * Filter MIDI messages to pitch bend changes only
     */
    function pitchBend() {
        return function (source) { return source.pipe(operators.filter(function (_a) {
            var data = _a.data;
            return between(data[0], 224, 239);
        })); };
    }

    /**
     * Filter MIDI messages to polyphonic aftertouch changes only
     */
    function polyphonicAftertouch() {
        return function (source) { return source.pipe(operators.filter(function (_a) {
            var data = _a.data;
            return between(data[0], 160, 175);
        })); };
    }

    /**
     * Filter MIDI messages to program changes only
     */
    function programChange() {
        return function (source) { return source.pipe(operators.filter(function (_a) {
            var data = _a.data;
            return between(data[0], 208, 223);
        })); };
    }

    /**
     * Filter MIDI messages to sustain pedal changes only
     */
    function sustainPedal() {
        return function (source) { return source.pipe(operators.filter(function (_a) {
            var data = _a.data;
            return between(data[0], 176, 191) && data[1] === 64;
        })); };
    }

    /**
     * Extract MIDI data from event
     */
    function toData() {
        return function (source) { return source.pipe(operators.map(function (_a) {
            var data = _a.data;
            return data;
        })); };
    }

    /**
     * Extract data byte (2nd) from MIDI message
     *
     * NOTE: Some status messages do not have 2nd byte, use it when you're certain
     */
    function toDataByte() {
        return function (source) { return source.pipe(operators.map(function (_a) {
            var data = _a.data;
            return data[1];
        })); };
    }

    /**
     * Extract status byte (1st) from MIDI message
     */
    function toStatusByte() {
        return function (source) { return source.pipe(operators.map(function (_a) {
            var data = _a.data;
            return data[0];
        })); };
    }

    /**
     * Extract received time from MIDI event
     */
    function toTimeStamp() {
        return function (source) { return source.pipe(operators.map(function (_a) {
            var timeStamp = _a.timeStamp;
            return timeStamp;
        })); };
    }

    /**
     * Extract value byte (3rd) from MIDI message
     *
     * NOTE: Some status messages do not have 3rd byte, use it when you're certain
     */
    function toValueByte() {
        return function (source) { return source.pipe(operators.map(function (_a) {
            var data = _a.data;
            return data[2];
        })); };
    }

    /**
     * Convert MIDI notes to frequencies
     *
     * @param note MIDI note
     * @param tuning tuning for middle A (440 by default)
     */
    function toFrequency(note, tuning) {
        if (tuning === void 0) { tuning = 440; }
        return Math.pow(2, (note - 69) / 12) * tuning;
    }

    var FrequencyPipe = /** @class */ (function () {
        function FrequencyPipe() {
        }
        FrequencyPipe.prototype.transform = function (note, tuning) {
            return toFrequency(note, tuning);
        };
        return FrequencyPipe;
    }());
    FrequencyPipe.ɵfac = i0__namespace.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0__namespace, type: FrequencyPipe, deps: [], target: i0__namespace.ɵɵFactoryTarget.Pipe });
    FrequencyPipe.ɵpipe = i0__namespace.ɵɵngDeclarePipe({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0__namespace, type: FrequencyPipe, name: "frequency" });
    i0__namespace.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0__namespace, type: FrequencyPipe, decorators: [{
                type: i0.Pipe,
                args: [{
                        name: 'frequency',
                    }]
            }] });

    var FrequencyPipeModule = /** @class */ (function () {
        function FrequencyPipeModule() {
        }
        return FrequencyPipeModule;
    }());
    FrequencyPipeModule.ɵfac = i0__namespace.ɵɵngDeclareFactory({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0__namespace, type: FrequencyPipeModule, deps: [], target: i0__namespace.ɵɵFactoryTarget.NgModule });
    FrequencyPipeModule.ɵmod = i0__namespace.ɵɵngDeclareNgModule({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0__namespace, type: FrequencyPipeModule, declarations: [FrequencyPipe], exports: [FrequencyPipe] });
    FrequencyPipeModule.ɵinj = i0__namespace.ɵɵngDeclareInjector({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0__namespace, type: FrequencyPipeModule });
    i0__namespace.ɵɵngDeclareClassMetadata({ minVersion: "12.0.0", version: "12.2.15", ngImport: i0__namespace, type: FrequencyPipeModule, decorators: [{
                type: i0.NgModule,
                args: [{
                        declarations: [FrequencyPipe],
                        exports: [FrequencyPipe],
                    }]
            }] });

    var SYSEX = new i0.InjectionToken('Require sysex MIDI access', {
        providedIn: 'root',
        factory: function () { return false; },
    });

    var MIDI_ACCESS = new i0.InjectionToken('Promise for MIDIAccess object', {
        providedIn: 'root',
        factory: function () {
            var navigatorRef = i0.inject(common.NAVIGATOR);
            var sysex = i0.inject(SYSEX);
            return navigatorRef.requestMIDIAccess
                ? navigatorRef.requestMIDIAccess({ sysex: sysex })
                : Promise.reject(new Error('Web MIDI API is not supported'));
        },
    });

    var MIDI_INPUT = new i0.InjectionToken('MIDIInput object');

    /******************************************************************************
    Copyright (c) Microsoft Corporation.

    Permission to use, copy, modify, and/or distribute this software for any
    purpose with or without fee is hereby granted.

    THE SOFTWARE IS PROVIDED "AS IS" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH
    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY
    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,
    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM
    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR
    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR
    PERFORMANCE OF THIS SOFTWARE.
    ***************************************************************************** */
    /* global Reflect, Promise */
    var extendStatics = function (d, b) {
        extendStatics = Object.setPrototypeOf ||
            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||
            function (d, b) { for (var p in b)
                if (Object.prototype.hasOwnProperty.call(b, p))
                    d[p] = b[p]; };
        return extendStatics(d, b);
    };
    function __extends(d, b) {
        if (typeof b !== "function" && b !== null)
            throw new TypeError("Class extends value " + String(b) + " is not a constructor or null");
        extendStatics(d, b);
        function __() { this.constructor = d; }
        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());
    }
    var __assign = function () {
        __assign = Object.assign || function __assign(t) {
            for (var s, i = 1, n = arguments.length; i < n; i++) {
                s = arguments[i];
                for (var p in s)
                    if (Object.prototype.hasOwnProperty.call(s, p))
                        t[p] = s[p];
            }
            return t;
        };
        return __assign.apply(this, arguments);
    };
    function __rest(s, e) {
        var t = {};
        for (var p in s)
            if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)
                t[p] = s[p];
        if (s != null && typeof Object.getOwnPropertySymbols === "function")
            for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {
                if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))
                    t[p[i]] = s[p[i]];
            }
        return t;
    }
    function __decorate(decorators, target, key, desc) {
        var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;
        if (typeof Reflect === "object" && typeof Reflect.decorate === "function")
            r = Reflect.decorate(decorators, target, key, desc);
        else
            for (var i = decorators.length - 1; i >= 0; i--)
                if (d = decorators[i])
                    r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;
        return c > 3 && r && Object.defineProperty(target, key, r), r;
    }
    function __param(paramIndex, decorator) {
        return function (target, key) { decorator(target, key, paramIndex); };
    }
    function __esDecorate(ctor, descriptorIn, decorators, contextIn, initializers, extraInitializers) {
        function accept(f) { if (f !== void 0 && typeof f !== "function")
            throw new TypeError("Function expected"); return f; }
        var kind = contextIn.kind, key = kind === "getter" ? "get" : kind === "setter" ? "set" : "value";
        var target = !descriptorIn && ctor ? contextIn["static"] ? ctor : ctor.prototype : null;
        var descriptor = descriptorIn || (target ? Object.getOwnPropertyDescriptor(target, contextIn.name) : {});
        var _, done = false;
        for (var i = decorators.length - 1; i >= 0; i--) {
            var context = {};
            for (var p in contextIn)
                context[p] = p === "access" ? {} : contextIn[p];
            for (var p in contextIn.access)
                context.access[p] = contextIn.access[p];
            context.addInitializer = function (f) { if (done)
                throw new TypeError("Cannot add initializers after decoration has completed"); extraInitializers.push(accept(f || null)); };
            var result = (0, decorators[i])(kind === "accessor" ? { get: descriptor.get, set: descriptor.set } : descriptor[key], context);
            if (kind === "accessor") {
                if (result === void 0)
                    continue;
                if (result === null || typeof result !== "object")
                    throw new TypeError("Object expected");
                if (_ = accept(result.get))
                    descriptor.get = _;
                if (_ = accept(result.set))
                    descriptor.set = _;
                if (_ = accept(result.init))
                    initializers.push(_);
            }
            else if (_ = accept(result)) {
                if (kind === "field")
                    initializers.push(_);
                else
                    descriptor[key] = _;
            }
        }
        if (target)
            Object.defineProperty(target, contextIn.name, descriptor);
        done = true;
    }
    ;
    function __runInitializers(thisArg, initializers, value) {
        var useValue = arguments.length > 2;
        for (var i = 0; i < initializers.length; i++) {
            value = useValue ? initializers[i].call(thisArg, value) : initializers[i].call(thisArg);
        }
        return useValue ? value : void 0;
    }
    ;
    function __propKey(x) {
        return typeof x === "symbol" ? x : "".concat(x);
    }
    ;
    function __setFunctionName(f, name, prefix) {
        if (typeof name === "symbol")
            name = name.description ? "[".concat(name.description, "]") : "";
        return Object.defineProperty(f, "name", { configurable: true, value: prefix ? "".concat(prefix, " ", name) : name });
    }
    ;
    function __metadata(metadataKey, metadataValue) {
        if (typeof Reflect === "object" && typeof Reflect.metadata === "function")
            return Reflect.metadata(metadataKey, metadataValue);
    }
    function __awaiter(thisArg, _arguments, P, generator) {
        function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }
        return new (P || (P = Promise))(function (resolve, reject) {
            function fulfilled(value) { try {
                step(generator.next(value));
            }
            catch (e) {
                reject(e);
            } }
            function rejected(value) { try {
                step(generator["throw"](value));
            }
            catch (e) {
                reject(e);
            } }
            function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }
            step((generator = generator.apply(thisArg, _arguments || [])).next());
        });
    }
    function __generator(thisArg, body) {
        var _ = { label: 0, sent: function () { if (t[0] & 1)
                throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;
        return g = { next: verb(0), "throw": verb(1), "return": verb(2) }, typeof Symbol === "function" && (g[Symbol.iterator] = function () { return this; }), g;
        function verb(n) { return function (v) { return step([n, v]); }; }
        function step(op) {
            if (f)
                throw new TypeError("Generator is already executing.");
            while (g && (g = 0, op[0] && (_ = 0)), _)
                try {
                    if (f = 1, y && (t = op[0] & 2 ? y["return"] : op[0] ? y["throw"] || ((t = y["return"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done)
                        return t;
                    if (y = 0, t)
                        op = [op[0] & 2, t.value];
                    switch (op[0]) {
                        case 0:
                        case 1:
                            t = op;
                            break;
                        case 4:
                            _.label++;
                            return { value: op[1], done: false };
                        case 5:
                            _.label++;
                            y = op[1];
                            op = [0];
                            continue;
                        case 7:
                            op = _.ops.pop();
                            _.trys.pop();
                            continue;
                        default:
                            if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) {
                                _ = 0;
                                continue;
                            }
                            if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) {
                                _.label = op[1];
                                break;
                            }
                            if (op[0] === 6 && _.label < t[1]) {
                                _.label = t[1];
                                t = op;
                                break;
                            }
                            if (t && _.label < t[2]) {
                                _.label = t[2];
                                _.ops.push(op);
                                break;
                            }
                            if (t[2])
                                _.ops.pop();
                            _.trys.pop();
                            continue;
                    }
                    op = body.call(thisArg, _);
                }
                catch (e) {
                    op = [6, e];
                    y = 0;
                }
                finally {
                    f = t = 0;
                }
            if (op[0] & 5)
                throw op[1];
            return { value: op[0] ? op[1] : void 0, done: true };
        }
    }
    var __createBinding = Object.create ? (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        var desc = Object.getOwnPropertyDescriptor(m, k);
        if (!desc || ("get" in desc ? !m.__esModule : desc.writable || desc.configurable)) {
            desc = { enumerable: true, get: function () { return m[k]; } };
        }
        Object.defineProperty(o, k2, desc);
    }) : (function (o, m, k, k2) {
        if (k2 === undefined)
            k2 = k;
        o[k2] = m[k];
    });
    function __exportStar(m, o) {
        for (var p in m)
            if (p !== "default" && !Object.prototype.hasOwnProperty.call(o, p))
                __createBinding(o, m, p);
    }
    function __values(o) {
        var s = typeof Symbol === "function" && Symbol.iterator, m = s && o[s], i = 0;
        if (m)
            return m.call(o);
        if (o && typeof o.length === "number")
            return {
                next: function () {
                    if (o && i >= o.length)
                        o = void 0;
                    return { value: o && o[i++], done: !o };
                }
            };
        throw new TypeError(s ? "Object is not iterable." : "Symbol.iterator is not defined.");
    }
    function __read(o, n) {
        var m = typeof Symbol === "function" && o[Symbol.iterator];
        if (!m)
            return o;
        var i = m.call(o), r, ar = [], e;
        try {
            while ((n === void 0 || n-- > 0) && !(r = i.next()).done)
                ar.push(r.value);
        }
        catch (error) {
            e = { error: error };
        }
        finally {
            try {
                if (r && !r.done && (m = i["return"]))
                    m.call(i);
            }
            finally {
                if (e)
                    throw e.error;
            }
        }
        return ar;
    }
    /** @deprecated */
    function __spread() {
        for (var ar = [], i = 0; i < arguments.length; i++)
            ar = ar.concat(__read(arguments[i]));
        return ar;
    }
    /** @deprecated */
    function __spreadArrays() {
        for (var s = 0, i = 0, il = arguments.length; i < il; i++)
            s += arguments[i].length;
        for (var r = Array(s), k = 0, i = 0; i < il; i++)
            for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)
                r[k] = a[j];
        return r;
    }
    function __spreadArray(to, from, pack) {
        if (pack || arguments.length === 2)
            for (var i = 0, l = from.length, ar; i < l; i++) {
                if (ar || !(i in from)) {
                    if (!ar)
                        ar = Array.prototype.slice.call(from, 0, i);
                    ar[i] = from[i];
                }
            }
        return to.concat(ar || Array.prototype.slice.call(from));
    }
    function __await(v) {
        return this instanceof __await ? (this.v = v, this) : new __await(v);
    }
    function __asyncGenerator(thisArg, _arguments, generator) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var g = generator.apply(thisArg, _arguments || []), i, q = [];
        return i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i;
        function verb(n) { if (g[n])
            i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }
        function resume(n, v) { try {
            step(g[n](v));
        }
        catch (e) {
            settle(q[0][3], e);
        } }
        function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }
        function fulfill(value) { resume("next", value); }
        function reject(value) { resume("throw", value); }
        function settle(f, v) { if (f(v), q.shift(), q.length)
            resume(q[0][0], q[0][1]); }
    }
    function __asyncDelegator(o) {
        var i, p;
        return i = {}, verb("next"), verb("throw", function (e) { throw e; }), verb("return"), i[Symbol.iterator] = function () { return this; }, i;
        function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: false } : f ? f(v) : v; } : f; }
    }
    function __asyncValues(o) {
        if (!Symbol.asyncIterator)
            throw new TypeError("Symbol.asyncIterator is not defined.");
        var m = o[Symbol.asyncIterator], i;
        return m ? m.call(o) : (o = typeof __values === "function" ? __values(o) : o[Symbol.iterator](), i = {}, verb("next"), verb("throw"), verb("return"), i[Symbol.asyncIterator] = function () { return this; }, i);
        function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }
        function settle(resolve, reject, d, v) { Promise.resolve(v).then(function (v) { resolve({ value: v, done: d }); }, reject); }
    }
    function __makeTemplateObject(cooked, raw) {
        if (Object.defineProperty) {
            Object.defineProperty(cooked, "raw", { value: raw });
        }
        else {
            cooked.raw = raw;
        }
        return cooked;
    }
    ;
    var __setModuleDefault = Object.create ? (function (o, v) {
        Object.defineProperty(o, "default", { enumerable: true, value: v });
    }) : function (o, v) {
        o["default"] = v;
    };
    function __importStar(mod) {
        if (mod && mod.__esModule)
            return mod;
        var result = {};
        if (mod != null)
            for (var k in mod)
                if (k !== "default" && Object.prototype.hasOwnProperty.call(mod, k))
                    __createBinding(result, mod, k);
        __setModuleDefault(result, mod);
        return result;
    }
    function __importDefault(mod) {
        return (mod && mod.__esModule) ? mod : { default: mod };
    }
    function __classPrivateFieldGet(receiver, state, kind, f) {
        if (kind === "a" && !f)
            throw new TypeError("Private accessor was defined without a getter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver))
            throw new TypeError("Cannot read private member from an object whose class did not declare it");
        return kind === "m" ? f : kind === "a" ? f.call(receiver) : f ? f.value : state.get(receiver);
    }
    function __classPrivateFieldSet(receiver, state, value, kind, f) {
        if (kind === "m")
            throw new TypeError("Private method is not writable");
        if (kind === "a" && !f)
            throw new TypeError("Private accessor was defined without a setter");
        if (typeof state === "function" ? receiver !== state || !f : !state.has(receiver))
            throw new TypeError("Cannot write private member to an object whose class did not declare it");
        return (kind === "a" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;
    }
    function __classPrivateFieldIn(state, receiver) {
        if (receiver === null || (typeof receiver !== "object" && typeof receiver !== "function"))
            throw new TypeError("Cannot use 'in' operator on non-object");
        return typeof state === "function" ? receiver === state : state.has(receiver);
    }

    function getPortsStream(ports) {
        return rxjs.from(i0.inject(MIDI_ACCESS).catch(function () { return null; })).pipe(operators.switchMap(function (access) { return access
            ? rxjs.fromEvent(access, 'statechange').pipe(operators.map(function () { return __spreadArray([], __read(access[ports].values())); }), operators.startWith(__spreadArray([], __read(access[ports].values()))))
            : rxjs.of([]); }), operators.shareReplay(1));
    }

    var MIDI_INPUTS = new i0.InjectionToken('Array of MIDI inputs', {
        factory: function () { return getPortsStream('inputs'); },
    });

    var MIDI_MESSAGES = new i0.InjectionToken('All incoming MIDI messages stream', {
        providedIn: 'root',
        factory: function () { return rxjs.from(i0.inject(MIDI_ACCESS).catch(function (e) { return e; })).pipe(operators.switchMap(function (access) { return access instanceof Error
            ? rxjs.throwError(access)
            : rxjs.fromEvent(access, 'statechange').pipe(operators.startWith(null), operators.switchMap(function () { return rxjs.merge.apply(void 0, __spreadArray([], __read(Array.from(access.inputs).map(function (_a) {
                var _b = __read(_a, 2), _ = _b[0], input = _b[1];
                return rxjs.fromEvent(input, 'midimessage');
            })))); })); }), operators.share()); },
    });

    var MIDI_OUTPUT = new i0.InjectionToken('MIDIOutput object');

    var MIDI_OUTPUTS = new i0.InjectionToken('Array of MIDI inputs', {
        factory: function () { return getPortsStream('outputs'); },
    });

    var MIDI_SUPPORT = new i0.InjectionToken('Web MIDI API support', {
        factory: function () { return !!i0.inject(common.NAVIGATOR).requestMIDIAccess; },
    });

    var MIDI_INPUT_QUERY = new i0.InjectionToken('MIDIInput object id or name');

    /**
     * Provide MIDIInput by id
     *
     * @param id
     */
    function inputById(id) {
        return [
            {
                provide: MIDI_INPUT_QUERY,
                useValue: id,
            },
            {
                provide: MIDI_INPUT,
                deps: [MIDI_ACCESS, MIDI_INPUT_QUERY],
                useFactory: inputByIdFactory,
            },
        ];
    }
    function inputByIdFactory(midiAccess, id) {
        return midiAccess.then(function (access) { return access.inputs.get(id); });
    }

    /**
     * Provide MIDIInput by name
     *
     * @param name
     */
    function inputByName(name) {
        return [
            {
                provide: MIDI_INPUT_QUERY,
                useValue: name,
            },
            {
                provide: MIDI_INPUT,
                deps: [MIDI_ACCESS, MIDI_INPUT_QUERY],
                useFactory: inputByNameFactory,
            },
        ];
    }
    function inputByNameFactory(midiAccess, name) {
        return midiAccess.then(function (access) { return __spreadArray([], __read(access.inputs.values())).find(function (input) { return input.name === name; }); });
    }

    var MIDI_OUTPUT_QUERY = new i0.InjectionToken('MIDIOutput object id or name');

    /**
     * Provide MIDIOutput by id
     *
     * @param id
     */
    function outputById(id) {
        return [
            {
                provide: MIDI_OUTPUT_QUERY,
                useValue: id,
            },
            {
                provide: MIDI_OUTPUT,
                deps: [MIDI_ACCESS, MIDI_OUTPUT_QUERY],
                useFactory: outputByIdFactory,
            },
        ];
    }
    function outputByIdFactory(midiAccess, id) {
        return midiAccess.then(function (access) { return access.outputs.get(id); });
    }

    /**
     * Provide MIDIOutput by name
     *
     * @param name
     */
    function outputByName(name) {
        return [
            {
                provide: MIDI_OUTPUT_QUERY,
                useValue: name,
            },
            {
                provide: MIDI_OUTPUT,
                deps: [MIDI_ACCESS, MIDI_OUTPUT_QUERY],
                useFactory: outputByNameFactory,
            },
        ];
    }
    function outputByNameFactory(midiAccess, name) {
        return midiAccess.then(function (access) { return __spreadArray([], __read(access.outputs.values())).find(function (output) { return output.name === name; }); });
    }

    var COEFFICIENT = Math.pow(2, (1 / 12));
    /**
     * Convert frequencies to MIDI notes
     *
     * @param frequency
     * @param tuning tuning for middle A (440 by default)
     */
    function toNote(frequency, tuning) {
        if (tuning === void 0) { tuning = 440; }
        return Math.round(Math.log(frequency / tuning) / Math.log(COEFFICIENT)) + 69;
    }

    /**
     * Public API Surface of @ng-web-apis/midi
     */

    /**
     * Generated bundle index. Do not edit.
     */

    exports.FrequencyPipe = FrequencyPipe;
    exports.FrequencyPipeModule = FrequencyPipeModule;
    exports.MIDI_ACCESS = MIDI_ACCESS;
    exports.MIDI_INPUT = MIDI_INPUT;
    exports.MIDI_INPUTS = MIDI_INPUTS;
    exports.MIDI_MESSAGES = MIDI_MESSAGES;
    exports.MIDI_OUTPUT = MIDI_OUTPUT;
    exports.MIDI_OUTPUTS = MIDI_OUTPUTS;
    exports.MIDI_SUPPORT = MIDI_SUPPORT;
    exports.SYSEX = SYSEX;
    exports.aftertouch = aftertouch;
    exports.filterByChannel = filterByChannel;
    exports.filterById = filterById;
    exports.filterByName = filterByName;
    exports.inputById = inputById;
    exports.inputByIdFactory = inputByIdFactory;
    exports.inputByName = inputByName;
    exports.inputByNameFactory = inputByNameFactory;
    exports.mainVolume = mainVolume;
    exports.modulationWheel = modulationWheel;
    exports.notes = notes;
    exports.outputById = outputById;
    exports.outputByIdFactory = outputByIdFactory;
    exports.outputByName = outputByName;
    exports.outputByNameFactory = outputByNameFactory;
    exports.pan = pan;
    exports.pitchBend = pitchBend;
    exports.polyphonicAftertouch = polyphonicAftertouch;
    exports.programChange = programChange;
    exports.sustainPedal = sustainPedal;
    exports.toData = toData;
    exports.toDataByte = toDataByte;
    exports.toFrequency = toFrequency;
    exports.toNote = toNote;
    exports.toStatusByte = toStatusByte;
    exports.toTimeStamp = toTimeStamp;
    exports.toValueByte = toValueByte;

    Object.defineProperty(exports, '__esModule', { value: true });

}));
//# sourceMappingURL=ng-web-apis-midi.umd.js.map
